@@ -34,8 +34,7 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: config.c,v 1.31 2008-02-24 10:33:49 al-guest Exp $")
+#include <limits.h>
 
 #include "port.h"
 #include "minicom.h"
@@ -53,11 +52,6 @@ void read_parms(void)
 
   /* Read global parameters */
   if ((fp = fopen(parfile, "r")) == NULL) {
-    if (real_uid == 0) {
-      fputs(_("minicom: WARNING: configuration file not found, using defaults\n"),stderr);
-      sleep(2);
-      return;
-    }
     fprintf(stderr,
             _("minicom: there is no global configuration file %s\n"), parfile);
     fputs(_("Ask your sysadmin to create one (with minicom -s).\n"), stderr);
@@ -66,7 +60,7 @@ void read_parms(void)
   readpars(fp, CONFIG_GLOBAL);
   fclose(fp);
   /* Read personal parameters */
-  if ((fp = sfopen(pparfile, "r")) != NULL) {
+  if ((fp = fopen(pparfile, "r")) != NULL) {
     readpars(fp, CONFIG_PERSONAL);
     fclose(fp);
   }
@@ -78,7 +72,7 @@ void read_parms(void)
 
   /* fmg - Read personal macros */
   if (P_MACROS[0] != 0) { /* fmg - null length? */
-    if ((fp = sfopen(pfix_home(P_MACROS), "r")) == NULL) {
+    if ((fp = fopen(pfix_home(P_MACROS), "r")) == NULL) {
       if (errno != ENOENT) {
         fprintf(stderr,
                 _("minicom: cannot open macro file %s\n"), pfix_home(P_MACROS));
@@ -1084,7 +1078,7 @@ static void doscrkeyb(void)
         pgets(w, mbslen (macros_file) + 1, 12, P_MACROS, 64, 64);
 
         /* Try to open the file to read it in. */
-        fp = sfopen(pfix_home(P_MACROS), "r+");
+        fp = fopen(pfix_home(P_MACROS), "r+");
         if (fp == NULL) {
           if (errno == EPERM) {
             /* Permission denied, hacker! */
@@ -1214,10 +1208,10 @@ int dotermmenu(void)
       case 'D':
         sprintf(buf, "%d", vt_nl_delay);
         mc_wlocate(w, mbslen(msg_nl_delay) +1, 4);
-        mc_wgets(w, buf, 4, 4);
+        mc_wgets(w, buf, 5, 5);
         vt_nl_delay = atoi(buf);
         mc_wlocate(w, mbslen(msg_nl_delay) +1, 4);
-        mc_wprintf(w, "%-3d", vt_nl_delay);
+        mc_wprintf(w, "%-4d", vt_nl_delay);
         break;
       case 'E':
         pgets(w, strlen(msg_answerback) + 1, 5, P_ANSWERBACK, 50, 50);
@@ -1243,29 +1237,17 @@ void vdodflsave(void)
 int dodflsave(void)
 {
   FILE *fp;
+  char *fname = dosetup ? parfile : pparfile;
 
-  /* Root saves new configuration */
-  if (real_uid == 0) {
-    if ((fp = fopen(parfile, "w")) == (FILE *)NULL) {
-      werror(_("Cannot write to %s"), parfile);
-      return -1;
-    }
-    writepars(fp, 1);
-    fclose(fp);
-    werror(_("Configuration saved"));
-  } else {
-    /* Mortals save their own configuration */
-    if ((fp = sfopen(pparfile, "w")) == (FILE *)NULL) {
-      werror(_("Cannot write to %s"), pparfile);
-      return -1;
-    }
-    writepars(fp, 0);
-    fclose(fp);
-    werror(_("Configuration saved"));
-  }
-  if (domacsave() < 0) /* fmg - something went wrong... */
+  if ((fp = fopen(fname, "w")) == NULL) {
+    werror(_("Cannot write to %s"), fname);
     return -1;
-  return 0;
+  }
+  writepars(fp, dosetup);
+  fclose(fp);
+  werror(_("Configuration saved"));
+
+  return domacsave() < 0 ? -1 : 0;
 }
 
 /*
@@ -1281,7 +1263,7 @@ int domacsave(void)
       werror(_("ERROR: Macros have changed but no filename is set!"));
       return -1;
     } else {
-      if ((fp = sfopen(pfix_home(P_MACROS), "w")) == (FILE *)NULL) {
+      if ((fp = fopen(pfix_home(P_MACROS), "w")) == (FILE *)NULL) {
         werror(_("Cannot write macros file %s"),
                pfix_home(P_MACROS));
         return -1;
@@ -1304,11 +1286,6 @@ static void donamsave(void)
   char ifile[128];
   char *s;
 
-  if (real_uid != 0) {
-    werror(_("You are not allowed to create a configuration"));
-    return;
-  }
-
   ifile[0] = 0;
   s = input(_("Give name to save this configuration?"), ifile);
   if (s != (char *)0 && *s != 0) {
@@ -1379,6 +1356,14 @@ static const unsigned int speeds[] =
    };
 #define NR_SPEEDS (sizeof(speeds) / sizeof(speeds[0]))
 
+int speed_valid(unsigned int speed)
+{
+  unsigned i;
+  for (i = 0; i < NR_SPEEDS; ++i)
+    if (speed == speeds[i])
+      return 1;
+  return 0;
+}
 
 /*
  * Ask user for Baudrate, Bits and Parity
@@ -1760,7 +1745,7 @@ void doconv(void)
         buf[0] = 0;
         i = -1;
         mc_wlocate(w, mbslen(prompt), ymax - 1);
-        mc_wgets(w, buf, 4, 4);
+        mc_wgets(w, buf, 7, 7);
         sscanf(buf, "%d", &i);
         if (i > 255 || i < 0) {
           werror(_("Input character ascii value 0-255"));
@@ -1772,7 +1757,7 @@ void doconv(void)
         prompt=_("Change input to: %s");
         mc_wprintf(w, prompt,buf);
         mc_wlocate(w, mbslen(prompt) + 28, ymax - 1);
-        mc_wgets(w, buf, 4, 4);
+        mc_wgets(w, buf, 7, 7);
         sscanf(buf, "%d", &j);
         if (j > 255 || j < 0) {
           werror(_("Input character ascii value 0-255"));
@@ -1784,7 +1769,7 @@ void doconv(void)
         prompt = _("Change output to: %s");
         mc_wprintf(w, prompt, buf);
         mc_wlocate(w, mbslen(prompt) + 52, ymax - 1);
-        mc_wgets(w, buf, 4, 4);
+        mc_wgets(w, buf, 7, 7);
         sscanf(buf, "%d", &j);
         if (j > 255 || j < 0) {
           werror(_("Input character ascii value 0-255"));
@@ -1818,7 +1803,7 @@ int loadconv(char *buf)
 {
   FILE *fp;
 
-  if ((fp = sfopen(pfix_home(buf), "rb")) == (FILE *)NULL) {
+  if ((fp = fopen(pfix_home(buf), "rb")) == (FILE *)NULL) {
     werror(_("Cannot open conversion table %s"), pfix_home(buf));
     return 1;
   }
@@ -1832,7 +1817,7 @@ int saveconv(char *buf)
 {
   FILE *fp;
 
-  if ((fp = sfopen(pfix_home(buf), "wb")) == (FILE *)NULL) {
+  if ((fp = fopen(pfix_home(buf), "wb")) == (FILE *)NULL) {
     werror(_("Cannot write conversion table %s"), pfix_home(buf));
     return 1;
   }
