@@ -31,11 +31,9 @@
 #endif
 
 #include <stdint.h>
+#include <limits.h>
 #include <arpa/inet.h>
 
-#include "rcsid.h"
-RCSID("$Id: dial.c,v 1.26 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -46,9 +44,6 @@ RCSID("$Id: dial.c,v 1.26 2007-10-10 20:18:20 al-guest Exp $")
 #    include <sys/kd.h>
 #    include <sys/time.h>
 #  endif
-#  if defined(_COH42) || defined(_SCO)
-#    include <sys/vtkd.h>
-#  endif
 #endif
 
 /* Dialing directory. */
@@ -353,7 +348,7 @@ void music(void)
     ioctl(consolefd, KIOCSOUND, k);
 
     /* Check keypress with timeout 160 ms */
-    x = check_io(-1, 0, 160, NULL, NULL);
+    x = check_io(-1, 0, 160, NULL, 0, NULL);
     if (x & 2)
       break;
   }
@@ -363,7 +358,7 @@ void music(void)
 
   /* Wait for keypress and absorb it */
   while ((x & 2) == 0) {
-    x = check_io(-1, 0, 10000, NULL, NULL);
+    x = check_io(-1, 0, 10000, NULL, 0, NULL);
     timer_update();
   }
   keyboard(KGETKEY, 0);
@@ -383,14 +378,14 @@ static int dialfailed(char *s, int rtime)
   mc_wprintf(dialwin, _("    No connection: %s.      \n"), s);
   if (rtime < 0) {
     mc_wprintf(dialwin, _("   Press any key to continue..    "));
-    if (check_io(-1, 0, 10000, NULL, NULL) & 2)
+    if (check_io(-1, 0, 10000, NULL, 0, NULL) & 2)
       keyboard(KGETKEY, 0);
     return 0;
   }
   mc_wprintf(dialwin, _("     Retry in %2d seconds             "), rtime);
 
   for (f = rtime - 1; f >= 0; f--) {
-    x = check_io(-1, 0, 1000, NULL, NULL);
+    x = check_io(-1, 0, 1000, NULL, 0, NULL);
     if (x & 2) {
       /* Key pressed - absorb it. */
       x = keyboard(KGETKEY, 0);
@@ -528,7 +523,7 @@ MainLoop:
     buf[0] = 0;
     while (dialtime > 0) {
       if (*s == 0) {
-        x = check_io(portfd_connected, 0, 1000, buf, NULL);
+        x = check_io(portfd_connected, 0, 1000, buf, sizeof(buf), NULL);
         s = buf;
       }
       if (x & 2) {
@@ -601,7 +596,7 @@ MainLoop:
             if (P_SOUND[0] == 'Y')
               music();
             else {
-              x = check_io(-1, 0, 0, NULL, NULL);
+              x = check_io(-1, 0, 0, NULL, 0, NULL);
               if ((x & 2) == 2)
                 keyboard(KGETKEY, 0);
             }
@@ -611,7 +606,7 @@ MainLoop:
             if (P_SOUND[0] == 'Y')
               mc_wputs(dialwin,"\007\007\007");
 #endif
-            x = check_io(-1, 0, 0, NULL, NULL);
+            x = check_io(-1, 0, 0, NULL, 0, NULL);
             if ((x & 2) == 2)
               keyboard(KGETKEY, 0);
           }
@@ -849,13 +844,14 @@ int readdialdir(void)
 
   /* Make the manual dial entry. */
   d_man = mkstdent();
-  strcpy(d_man->name, _("Manually entered number"));
+  strncpy(d_man->name, _("Manually entered number"), sizeof(d_man->name));
+  d_man->name[sizeof(d_man->name) - 1] = 0;
 
   /* Construct path */
   snprintf(dfile, sizeof(dfile), "%s/.dialdir", homedir);
 
   /* Try to open ~/.dialdir */
-  if ((fp = sfopen(dfile, "r")) == NULL) {
+  if ((fp = fopen(dfile, "r")) == NULL) {
     if (errno == EPERM) {
       werror(_("Cannot open ~/.dialdir: permission denied"));
       dialents = mkstdent();
@@ -1025,7 +1021,7 @@ static void writedialdir(void)
   snprintf(dfile, sizeof(dfile), "%s/.dialdir", homedir);
 
   omask = umask(077);
-  if ((fp = sfopen(dfile, "w")) == NULL) {
+  if ((fp = fopen(dfile, "w")) == NULL) {
     umask(omask);
     werror(_("Cannot open ~/.dialdir for writing!"));
     dendd = 1;
