@@ -24,8 +24,7 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: file.c,v 1.13 2007-10-10 20:18:20 al-guest Exp $")
+#include <limits.h>
 
 #include "port.h"
 #include "minicom.h"
