@@ -22,9 +22,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: ipc.c,v 1.12 2007-10-10 20:18:20 al-guest Exp $")
-
 #include <strings.h>
 
 #include "port.h"
@@ -32,7 +29,8 @@ RCSID("$Id: ipc.c,v 1.12 2007-10-10 20:18:20 al-guest Exp $")
 #include "intl.h"
 
 /* Check if there is IO pending. */
-int check_io(int fd1, int fd2, int tmout, char *buf, int *buflen)
+int check_io(int fd1, int fd2, int tmout, char *buf,
+             int bufsize, int *bytes_read)
 {
   int n = 0, i;
   struct timeval tv;
@@ -60,15 +58,15 @@ int check_io(int fd1, int fd2, int tmout, char *buf, int *buflen)
   if (buf) {
     i = 0;
     if ((n & 1) == 1) {
-      i = read(fd1, buf, 127);
+      i = read(fd1, buf, bufsize - 1);
 #ifdef USE_SOCKET
       if (!i && portfd_is_socket && portfd == fd1)
         term_socket_close();
 #endif /* USE_SOCKET */
     }
     buf[i > 0 ? i : 0] = 0;
-    if (buflen)
-      *buflen = i;
+    if (bytes_read)
+      *bytes_read = i;
   }
 
   return n;
