@@ -23,9 +23,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: main.c,v 1.23 2008-02-02 22:49:53 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -85,13 +82,9 @@ void leave(const char *s)
     m_restorestate(portfd);
     close(portfd);
   }
-  set_privs();
-  if (lockfile[0])
-    unlink(lockfile);
+  lockfile_remove();
   if (P_CALLIN[0])
     fastsystem(P_CALLIN, NULL, NULL, NULL);
-  if (real_uid)
-    chown(dial_tty, (uid_t)portuid, (gid_t)portgid);
   fprintf(stderr, "%s", s);
   exit(1);
 }
@@ -107,7 +100,7 @@ char *esc_key(void)
     sprintf(buf, "CTRL-%c ", P_ESCAPE[1]);
     return buf;
   }
-#if defined(_COHERENT) || defined(i386) || defined(__i386__)
+#if defined(i386) || defined(__i386__)
   sprintf(buf, "ESC,");
 #else
   sprintf(buf, "Meta-");
@@ -164,14 +157,10 @@ int open_term(int doinit, int show_win_on_error)
   char buf[128];
   int fd, n = 0;
   int pid;
-  int mask;
 #ifdef HAVE_ERRNO_H
   int s_errno;
 #endif
 
-  /* Upgrade our status. */
-  set_privs();
-
   /* First see if the lock file directory is present. */
   if (P_LOCK[0] && stat(P_LOCK, &stt) == 0) {
 
@@ -216,32 +205,12 @@ int open_term(int doinit, int show_win_on_error)
       if (stdwin)
         mc_wclose(stdwin, 1);
       fprintf(stderr, _("Device %s is locked.\n"), dial_tty);
-      drop_privs();
       return -1;
     }
   }
 
-  if (doinit > 0 && lockfile[0]) {
-    /* Create lockfile compatible with UUCP-1.2 */
-    mask = umask(022);
-#ifdef _COH3
-    if ((fd = creat(lockfile, 0666)) < 0) {
-#else
-      if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
-#endif
-        if (stdwin)
-          mc_wclose(stdwin, 1);
-        fprintf(stderr, _("Cannot create lockfile. Sorry.\n"));
-        drop_privs();
-        return -1;
-      }
-      umask(mask);
-      chown(lockfile, (uid_t)real_uid, (gid_t)real_gid);
-      snprintf(buf, sizeof(buf), "%10ld minicom %.20s\n",
-                                 (long)getpid(), username);
-      write(fd, buf, strlen(buf));
-      close(fd);
-  }
+  if (doinit > 0)
+    lockfile_create();
 
   /* Run a special program to disable callin if needed. */
     if (doinit > 0 && P_CALLOUT[0]) {
@@ -249,9 +218,7 @@ int open_term(int doinit, int show_win_on_error)
         if (stdwin)
           mc_wclose(stdwin, 1);
         fprintf(stderr, _("Could not setup for dial out.\n"));
-        if (lockfile[0])
-          unlink(lockfile);
-        drop_privs();
+	lockfile_remove();
         return -1;
       }
     }
@@ -309,28 +276,15 @@ int open_term(int doinit, int show_win_on_error)
 #else
       fprintf(stderr, _("minicom: cannot open %s. Sorry.\n"), dial_tty);
 #endif
-      if (lockfile[0])
-        unlink(lockfile);
-      drop_privs();
+      lockfile_remove();
       return -1;
     }
-    if (lockfile[0])
-      unlink(lockfile);
+    lockfile_remove();
     if (show_win_on_error)
       werror(_("Cannot open %s!"), dial_tty);
-    drop_privs();
     return -1;
   }
 
-  /* Remember owner of port */
-  stat(dial_tty, &stt);
-  portuid = stt.st_uid;
-  portgid = stt.st_gid;
-
-  /* Give it to us! */
-  if (real_uid != 0)
-    chown(dial_tty, (uid_t)real_uid, (gid_t)real_gid);
-
   /* Set CLOCAL mode */
   m_nohang(portfd);
 
@@ -338,7 +292,6 @@ int open_term(int doinit, int show_win_on_error)
   m_hupcl(portfd, 1);
   if (doinit > 0)
     m_flush(portfd);
-  drop_privs();
   return 0;
 }
 
@@ -651,6 +604,7 @@ static void showtemp(void)
 int do_terminal(void)
 {
   char buf[128];
+  int buf_offset = 0;
   char *ptr;
   int c;
   int x;
@@ -710,22 +664,45 @@ dirty_goto:
     /* check if device is ok, if not, try to open it */
     if (!get_device_status(portfd_connected)) {
       if (open_term(0, 0) < 0) {
-	if (!error_on_open_window)
+        if (!error_on_open_window)
           error_on_open_window = mc_tell(_("Cannot open %s!"), dial_tty);
       } else {
-	if (error_on_open_window) {
+        if (error_on_open_window) {
           mc_wclose(error_on_open_window, 1);
-	  error_on_open_window = NULL;
-	}
+          error_on_open_window = NULL;
+        }
       }
     }
 
     /* Check for I/O or timer. */
-    x = check_io(portfd_connected, 0, 1000, buf, &blen);
+    x = check_io(portfd_connected, 0, 1000,
+                 buf + buf_offset, sizeof(buf) - buf_offset, &blen);
+    blen += buf_offset;
+    buf_offset = 0;
 
-    /*  Send data from the modem to the screen. */
+    /* Data from the modem to the screen. */
     if ((x & 1) == 1) {
-      ptr = buf;
+
+      if (using_iconv()) {
+        char obuf[sizeof(buf)];
+        char *otmp = obuf;
+        size_t output_len = sizeof(obuf);
+        size_t input_len = blen;
+
+        ptr = buf;
+        do_iconv(&ptr, &input_len, &otmp, &output_len);
+
+        if (input_len) { // something remained, we need to adapt buf accordingly
+          memmove(buf, ptr, input_len);
+          buf_offset = input_len;
+        }
+
+        blen = sizeof(obuf) - output_len;
+        ptr = obuf;
+      } else {
+        ptr = buf;
+      }
+
       while (blen-- > 0) {
         /* Auto zmodem detect */
         if (zauto) {
