@@ -27,12 +27,11 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID(PKG_VER "$Id: minicom.c,v 1.34 2008-02-02 22:49:53 al-guest Exp $")
-
 #include <getopt.h>
 #include <wchar.h>
 #include <wctype.h>
+#include <iconv.h>
+#include <limits.h>
 
 #define EXTERN
 #include "port.h"
@@ -48,10 +47,6 @@ static const char option_string[] =
 #define RESET 1
 #define NORESET 2
 
-#ifdef _SVR2
-extern struct passwd *getpwuid();
-#endif /*_SVR2*/
-
 #ifdef DEBUG
 /* Show signals when debug is on. */
 static void signore(int sig)
@@ -152,9 +147,6 @@ static void shjump(int sig)
       signal(f, SIG_DFL);
     for (f = 3; f < 20; f++)
       close(f);
-    set_privs();
-    setgid((gid_t)real_gid);
-    setuid((uid_t)real_uid);
     fputs(_("Shelled out. Type \"exit\" to return.\n"), stdout);
     execl(sh, sh, NULL);
     exit(1);
@@ -167,7 +159,7 @@ static void shjump(int sig)
 #endif /*SIGTSTP*/
 
 /* Get a line from either window or scroll back buffer. */
-static ELM *getline(WIN *w, int no)
+static ELM *mc_getline(WIN *w, int no)
 {
   int i;
   static ELM outofrange[MAXCOLS] = {{0,0,0}};
@@ -205,7 +197,7 @@ static void drawhist(WIN *w, int y, int r)
 
   w->direct = 0;
   for (f = 0; f < w->ys; f++)
-    mc_wdrawelm(w, f, getline(w, y++));
+    mc_wdrawelm(w, f, mc_getline(w, y++));
   if (r)
     mc_wredraw(w, 1);
   w->direct = 1;
@@ -227,7 +219,7 @@ void drawhist_look(WIN *w, int y, int r, wchar_t *look, int case_matters)
   tmp_line[0]='\0';
   w->direct = 0;
   for (f = 0; f < w->ys; f++) {
-    tmp_e = getline(w, y++);
+    tmp_e = mc_getline(w, y++);
 
     /* First we "accumulate" the line into a variable */
     mc_wdrawelm_var(w, tmp_e, tmp_line);
@@ -324,7 +316,7 @@ int find_next(WIN *w, WIN *w_hist,
 
   for (next_line = hit_line; next_line <= all_lines; next_line++) {
     /* we do 'something' here... :-) */
-    tmp_e = getline(w_hist, next_line);
+    tmp_e = mc_getline(w_hist, next_line);
 
     /*
      * First we "accumulate" the line into a variable.
@@ -379,9 +371,9 @@ wchar_t *StrStr(wchar_t *str1, wchar_t *str2, int case_matters)
 static void drawcite(WIN *w, int y, int citey, int start, int end)
 {
   if (y+citey >= start && y+citey <= end)
-    mc_wdrawelm_inverse(w, y, getline(w, y+citey));
+    mc_wdrawelm_inverse(w, y, mc_getline(w, y+citey));
   else
-    mc_wdrawelm(w, y, getline(w, y+citey));
+    mc_wdrawelm(w, y, mc_getline(w, y+citey));
 }
 
 static void drawcite_whole(WIN *w, int y, int start, int end)
@@ -400,7 +392,7 @@ static void do_cite(WIN *w, int start, int end)
   for (y=start; y<=end; y++) {
     vt_send('>');
     vt_send(' ');
-    tmp_e = getline(w, y);
+    tmp_e = mc_getline(w, y);
     mc_wdrawelm_var(w, tmp_e, tmp_line);
     tmp_line[w->xs] = 0;
     for (x = w->xs-1; x >= 0; x--) {
@@ -600,7 +592,7 @@ static void scrollback(void)
         if (citemode) {
           inverse = (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
         } else {
-          tmp_e = getline(b_us, y);
+          tmp_e = mc_getline(b_us, y);
           if (wcslen(look_for) > 1) {
             /* quick scan for pattern match */
             mc_wdrawelm_var(b_us, tmp_e, tmp_line);
@@ -611,9 +603,9 @@ static void scrollback(void)
         }
 
         if (inverse)
-          mc_wdrawelm_inverse(b_us, 0, getline(b_us, y));
+          mc_wdrawelm_inverse(b_us, 0, mc_getline(b_us, y));
         else
-          mc_wdrawelm(b_us, 0, getline(b_us, y));
+          mc_wdrawelm(b_us, 0, mc_getline(b_us, y));
         if (citemode)
           mc_wlocate(b_us, 0, cite_y);
         mc_wflush();
@@ -648,7 +640,7 @@ static void scrollback(void)
         if (citemode) {
           inverse = (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
         } else {
-          tmp_e = getline(b_us, y + b_us->ys - 1);
+          tmp_e = mc_getline(b_us, y + b_us->ys - 1);
           if (wcslen(look_for) > 1) {
             /* quick scan for pattern match */
             mc_wdrawelm_var(b_us, tmp_e, tmp_line);
@@ -660,10 +652,10 @@ static void scrollback(void)
 
         if (inverse)
           mc_wdrawelm_inverse(b_us, b_us->ys - 1,
-                           getline(b_us, y + b_us->ys - 1));
+                           mc_getline(b_us, y + b_us->ys - 1));
         else
           mc_wdrawelm(b_us, b_us->ys - 1,
-                   getline(b_us, y + b_us->ys - 1));
+                   mc_getline(b_us, y + b_us->ys - 1));
         if (citemode)
           mc_wlocate(b_us, 0, cite_y);
         mc_wflush();
@@ -753,7 +745,7 @@ static void scrollback(void)
         mc_wlocate(b_st, 0, 0);
         mc_wprintf(b_st, hline);
         mc_wredraw(b_st, 1);
-        mc_wdrawelm_inverse(b_us, cite_y, getline(b_us, cite_ystart));
+        mc_wdrawelm_inverse(b_us, cite_y, mc_getline(b_us, cite_ystart));
         mc_wlocate(b_us, 0, cite_y);
         break;
       case K_ESC:
@@ -797,32 +789,6 @@ static void change_size(int sig)
 }
 #endif /*SIGWINCH*/
 
-/*
- * Read a word from strings 's' and advance pointer.
- */
-static char *getword(char **s)
-{
-  char *begin;
-
-  /* Skip space */
-  while (**s == ' ' || **s == '\t')
-    (*s)++;
-  /* End of line? */
-  if (**s == '\0' || **s == '\n')
-    return NULL;
-
-  begin = *s;
-  /* Skip word */
-  while (**s != ' ' && **s != '\t' && **s != '\n' && **s)
-    (*s)++;
-  /* End word with '\0' */
-  if (**s) {
-    **s = 0;
-    (*s)++;
-  }
-  return begin;
-}
-
 static void usage(int env_args, int optind, char *mc)
 {
   if (env_args >= optind && mc)
@@ -839,7 +805,9 @@ static void helpthem(void)
   printf(_(
     "Usage: %s [OPTION]... [configuration]\n"
     "A terminal program for Linux and other unix-like systems.\n\n"
-    "  -s, --setup            : enter setup mode (only as root)\n"
+    "  -b, --baudrate         : set baudrate (ignore the value from config)\n"
+    "  -D, --device           : set device name (ignore the value from config)\n"
+    "  -s, --setup            : enter setup mode\n"
     "  -o, --noinit           : do not initialize modem & lockfiles at startup\n"
     "  -m, --metakey          : use meta or alt key for commands\n"
     "  -M, --metakey8         : use 8bit meta key for commands\n"
@@ -886,18 +854,87 @@ void toggle_local_echo(void)
   vt_set(-1, -1, -1, -1, local_echo, -1 ,-1);
 }
 
+
+/* -------------------------------------------- */
+
+static iconv_t iconv_rem2local;
+static int iconv_enabled;
+
+int using_iconv(void)
+{
+  return iconv_enabled;
+}
+
+static void init_iconv(const char *remote_charset)
+{
+  char local_charset[40];
+  char *tmp;
+
+  if (!remote_charset)
+    return;
+
+  // try to find some charset that minicom is using
+  tmp = getenv("LC_CTYPE");
+  if (!tmp)
+    tmp = getenv("LC_ALL");
+  if (!tmp)
+    tmp = getenv("LANG");
+  if (!tmp)
+    tmp = "ASCII";
+
+  // should be like 'en_US.UTF-8'
+  if (strchr(tmp, '.'))
+    tmp = strchr(tmp, '.') + 1;
+
+  snprintf(local_charset, sizeof(local_charset),
+           "%s//TRANSLIT", tmp);
+  local_charset[sizeof(local_charset) - 1] = 0;
+
+  //printf("Remote charset: %s\n", remote_charset);
+  //printf("Local charset: %s\n", local_charset);
+
+  iconv_rem2local = iconv_open(local_charset, remote_charset);
+  if (iconv_rem2local != (iconv_t)-1)
+    iconv_enabled = 1;
+  else
+    printf("Activating iconv failed with: %s(%d)\n",
+           strerror(errno), errno);
+}
+
+void do_iconv(char **inbuf, size_t *inbytesleft,
+              char **outbuf, size_t *outbytesleft)
+{
+  if (!iconv_enabled
+      || iconv(iconv_rem2local,
+               inbuf, inbytesleft,
+               outbuf, outbytesleft) == (size_t)-1) {
+    memcpy(*outbuf, *inbuf,
+           *outbytesleft < *inbytesleft ? *outbytesleft : *inbytesleft);
+
+    // in case of error re-init conversion descriptor
+    // (will this work?)
+    if (iconv_enabled)
+      iconv(iconv_rem2local, NULL, NULL, NULL, NULL);
+    return;
+  }
+}
+
+static void close_iconv(void)
+{
+  if (iconv_enabled)
+    iconv_close(iconv_rem2local);
+}
+/* -------------------------------------------- */
+
+
 int main(int argc, char **argv)
 {
   int c;			/* Command character */
   int quit = 0;			/* 'q' or 'x' pressed */
   char *s, *bufp;		/* Scratch pointers */
-  int dosetup = 0, doinit = 1;	/* -o and -s options */
-  char buf[80];			/* Keyboard input buffer */
+  int doinit = 1;		/* -o option */
   char capname[128];		/* Name of capture file */
   struct passwd *pwd;		/* To look up user name */
-  int userok = 0;		/* Scratch variables */
-  FILE *fp;			/* Scratch file pointer */
-  char userfile[256];		/* Locate user file */
   char *use_port;		/* Name of initialization file */
   char *args[20];		/* New argv pointer */
   int argk = 1;			/* New argc */
@@ -905,31 +942,37 @@ int main(int argc, char **argv)
   int env_args;			/* Number of args in env. variable */
   char *cmd_dial;		/* Entry from the command line. */
   int alt_code = 0;		/* Type of alt key */
+  char *cmdline_baudrate = NULL;/* Baudrate given on the command line via -b */
+  char *cmdline_device = NULL;  /* Device/Port given on the command line via -D */
+  char *remote_charset = NULL;  /* Remote charset given on the command line via -R */
   char pseudo[64];
   /* char* console_encoding = getenv ("LC_CTYPE"); */
 
   static struct option long_options[] =
   {
-    { "setup",       no_argument,       NULL, 's' },
-    { "help",        no_argument,       NULL, 'h' },
-    { "ptty",        required_argument, NULL, 'p' },
-    { "metakey",     no_argument,       NULL, 'm' },
-    { "metakey8",    no_argument,       NULL, 'M' },
-    { "ansi",        no_argument,       NULL, 'l' },
-    { "iso",         no_argument,       NULL, 'L' },
-    { "term",        required_argument, NULL, 't' },
-    { "noinit",      no_argument,       NULL, 'o' },
-    { "color",       required_argument, NULL, 'c' },
-    { "attrib",      required_argument, NULL, 'a' },
-    { "dial",        required_argument, NULL, 'd' },
-    { "statline",    no_argument,       NULL, 'z' },
-    { "capturefile", required_argument, NULL, 'C' },
-    { "script",      required_argument, NULL, 'S' },
-    { "7bit",        no_argument,       NULL, '7' },
-    { "8bit",        no_argument,       NULL, '8' },
-    { "version",     no_argument,       NULL, 'v' },
-    { "wrap",        no_argument,       NULL, 'w' },
-    { "disabletime", no_argument,       NULL, 'T' },
+    { "setup",         no_argument,       NULL, 's' },
+    { "help",          no_argument,       NULL, 'h' },
+    { "ptty",          required_argument, NULL, 'p' },
+    { "metakey",       no_argument,       NULL, 'm' },
+    { "metakey8",      no_argument,       NULL, 'M' },
+    { "ansi",          no_argument,       NULL, 'l' },
+    { "iso",           no_argument,       NULL, 'L' },
+    { "term",          required_argument, NULL, 't' },
+    { "noinit",        no_argument,       NULL, 'o' },
+    { "color",         required_argument, NULL, 'c' },
+    { "attrib",        required_argument, NULL, 'a' },
+    { "dial",          required_argument, NULL, 'd' },
+    { "statline",      no_argument,       NULL, 'z' },
+    { "capturefile",   required_argument, NULL, 'C' },
+    { "script",        required_argument, NULL, 'S' },
+    { "7bit",          no_argument,       NULL, '7' },
+    { "8bit",          no_argument,       NULL, '8' },
+    { "version",       no_argument,       NULL, 'v' },
+    { "wrap",          no_argument,       NULL, 'w' },
+    { "disabletime",   no_argument,       NULL, 'T' },
+    { "baudrate",      required_argument, NULL, 'b' },
+    { "device",        required_argument, NULL, 'D' },
+    { "remotecharset", required_argument, NULL, 'R' },
     { NULL, 0, NULL, 0 }
   };
 
@@ -973,10 +1016,6 @@ int main(int argc, char **argv)
   size_changed = 0;
   escape = 1;
   cmd_dial = NULL;
-  real_uid = getuid();
-  real_gid = getgid();
-  eff_uid  = geteuid();
-  eff_gid  = getegid();
 
   /* fmg 1/11/94 colors (set defaults) */
   /* MARK updated 02/17/95 to be more similiar to TELIX */
@@ -1009,6 +1048,7 @@ int main(int argc, char **argv)
    */
   args[0] = "minicom";
   if ((mc = getenv("MINICOM")) != NULL) {
+    char buf[80];
     strncpy(buf, mc, 80);
     bufp = buf;
     buf[79] = 0;
@@ -1031,7 +1071,7 @@ int main(int argc, char **argv)
 
   do {
     /* Process options with getopt */
-    while ((c = getopt_long(argk, args, "v78zhlLsomMbwTc:a:t:d:p:C:S:",
+    while ((c = getopt_long(argk, args, "v78zhlLsomMb:wTc:a:t:d:p:C:S:D:R:",
                             long_options, NULL)) != EOF)
       switch(c) {
 	case 'v':
@@ -1047,11 +1087,7 @@ int main(int argc, char **argv)
                  "2 of the License, or (at your option) any later version.\n\n");
           exit(1);
           break;
-        case 's': /* setup */
-          if (real_uid != 0 && real_uid != eff_uid) {
-            fprintf(stderr, _("minicom: -s switch needs root privilige\n"));
-            exit(1);
-          }
+        case 's': /* setup mode */
           dosetup = 1;
           break;
         case 'h':
@@ -1072,9 +1108,6 @@ int main(int argc, char **argv)
           }
           snprintf(pseudo, sizeof(pseudo), "/dev/%s", optarg);
           dial_tty = pseudo;
-
-          /* Drop priviliges. */
-          drop_all_privs();
           break;
         case 'm': /* ESC prefix metakey */
           alt_override++;
@@ -1132,7 +1165,7 @@ int main(int argc, char **argv)
           use_status = 1;
           break;
         case 'C': /* Capturing */
-          capfp = sfopen(optarg, "a");
+          capfp = fopen(optarg, "a");
           if (capfp == NULL) {
             werror(_("Cannot open capture file"));
             exit(1);
@@ -1155,6 +1188,15 @@ int main(int argc, char **argv)
         case 'T': /* disable online time */
           disable_online_time = 1;
           break;
+	case 'b':
+	  cmdline_baudrate = optarg;
+	  break;
+	case 'D':
+	  cmdline_device = optarg;
+	  break;
+	case 'R':
+	  remote_charset = optarg;
+	  break;
         default:
           usage(env_args, optind, mc);
           break;
@@ -1167,22 +1209,12 @@ int main(int argc, char **argv)
     /* Loop again if more options */
   } while (optind < argk);
 
+  init_iconv(remote_charset);
+
   if (screen_iso && screen_ibmpc)
     /* init VT */
     vt_set(-1, -1, -1, -1, -1, -1, 1);
 
-
-
-#ifdef SVR4_LOCKS
-  if (real_uid == 0 && dosetup == 0) {
-    fprintf(stderr, "%s",
-            _("minicom: WARNING: please don't run minicom as root when not maintaining\n"
-              "                  it (with the -s switch) since all changes to the\n"
-              "                  configuration will be GLOBAL !.\n"));
-    sleep(5);
-  }
-#endif
-
   /* Avoid fraude ! */	
   for (s = use_port; *s; s++)
     if (*s == '/')
@@ -1190,7 +1222,7 @@ int main(int argc, char **argv)
   snprintf(parfile, sizeof(parfile), "%s/minirc.%s", CONFDIR, use_port);
 
   /* Get password file information of this user. */
-  if ((pwd = getpwuid(real_uid)) == (struct passwd *)0) {
+  if ((pwd = getpwuid(getuid())) == NULL) {
     fputs(_("You don't exist. Go away.\n"), stderr);
     exit(1);
   }
@@ -1205,42 +1237,6 @@ int main(int argc, char **argv)
   /* Get personal parameter file */
   snprintf(pparfile, sizeof(pparfile), "%s/.minirc.%s", homedir, use_port);
 
-  /* Check this user in the USERFILE */
-  if (real_uid != 0 && real_uid != eff_uid) {
-    snprintf(userfile, sizeof(userfile), "%s/minicom.users", CONFDIR);
-    if ((fp = fopen(userfile, "r")) != (FILE *)0) {
-      while (fgets(buf, 70, fp) != NULL && !userok) {
-        /* Read first word */
-        bufp = buf;
-        s = getword(&bufp);
-        /* See if the "use_port" matches */
-        if (s && (!strcmp(pwd->pw_name, s) ||
-              strcmp("ALL", s) == 0)) {
-          if ((s = getword(&bufp)) == NULL)
-            userok = 1;
-          else
-            do {
-              if (!strcmp(s, use_port)) {
-                userok = 1;
-                break;
-              }
-            } while ((s = getword(&bufp)) != NULL);
-        }
-      }
-      fclose(fp);
-      if (!userok) {
-        fprintf(stderr,
-                _("Sorry %s. You are not allowed to use configuration %s.\n"),
-                pwd->pw_name, use_port);
-        fprintf(stderr,
-                _("Ask your sysadmin to add your name to %s\n"),
-                userfile);
-        exit(1);
-      }
-    }
-  }
-  buf[0] = 0;
-
   read_parms();
   num_hist_lines = atoi(P_HISTSIZE);
   strcpy(logfname,P_LOGFNAME);
@@ -1249,6 +1245,22 @@ int main(int argc, char **argv)
   addlf      = strcasecmp(P_ADDLINEFEED, "yes") == 0;
   local_echo = strcasecmp(P_LOCALECHO,   "yes") == 0;
 
+  /* After reading in the config via read_parms we can possibly overwrite
+   * the baudrate with a value given at the cmdline */
+  if (cmdline_baudrate) {
+    unsigned int b = strtol(cmdline_baudrate, (char **)NULL, 0);
+    if (speed_valid(b)) {
+      snprintf(P_BAUDRATE, sizeof(P_BAUDRATE), "%d", b);
+      P_BAUDRATE[sizeof(P_BAUDRATE) - 1] = 0;
+    }
+  }
+
+  /* New we can also overwrite the device name, if one was given */
+  if (cmdline_device) {
+    strncpy(P_PORT, cmdline_device, sizeof(P_PORT));
+    P_PORT[sizeof(P_PORT) - 1] = 0;
+  }
+
   stdwin = NULL; /* It better be! */
 
   /* Reset colors if we don't use 'em. */
@@ -1351,12 +1363,11 @@ int main(int argc, char **argv)
 #if defined (__DATE__) && defined (__TIME__)
   mc_wprintf(us, "%s %s, %s.\r\n",_("Compiled on"), __DATE__,__TIME__);
 #endif
-  mc_wprintf(us, "Port %s\n", P_PORT);
+  mc_wprintf(us, "Port %s\r\n", P_PORT);
+  if (using_iconv())
+    mc_wprintf(us, _("Using character set conversion\r\n"));
   mc_wprintf(us, _("\nPress %sZ for help on special keys%c\n\n"),esc_key(),'\r');
 
-  /* Now that all initialization is done, drop our priviliges. */
-  drop_privs();
-
   readdialdir();
 
   if (scr_name[0])
@@ -1439,7 +1450,7 @@ dirty_goto:
           s = input(_("Capture to which file? "), capname);
           if (s == NULL || *s == 0)
             break;
-          if ((capfp = sfopen(s, "a")) == (FILE *)NULL) {
+          if ((capfp = fopen(s, "a")) == (FILE *)NULL) {
             werror(_("Cannot open capture file"));
             break;
           }
@@ -1542,17 +1553,15 @@ dirty_goto:
   mc_wclose(us, 0);
   mc_wclose(st, 0);
   mc_wclose(stdwin, 1);
-  set_privs();
   keyboard(KUNINSTALL, 0);
   if (lockfile[0])
     unlink(lockfile);
   close(portfd);
-  /* Please - if your system doesn't have uid_t and/or gid_t, define 'em
-   * conditionally in "port.h".
-   */
-  chown(dial_tty, (uid_t)portuid, (gid_t)portgid);
+
   if (quit != NORESET && P_CALLIN[0])
     fastsystem(P_CALLIN, NULL, NULL, NULL);
 
+  close_iconv();
+
   return 0;
 }
