@@ -1,7 +1,7 @@
 /*
  * minicom.h	Constants, defaults, globals etc.
  *
- *		$Id: minicom.h,v 1.22 2008-02-02 22:49:53 al-guest Exp $
+ *		$Id: minicom.h,v 1.26 2009-06-06 21:19:36 al-guest Exp $
  *
  *		This file is part of the minicom communications package,
  *		Copyright 1991-1995 Miquel van Smoorenburg.
@@ -58,15 +58,13 @@ EXTERN int LINES;
 EXTERN int COLS;
 #endif
 
+EXTERN int dosetup;
+
 EXTERN char stdattr;	/* Standard attribute */
 
 EXTERN WIN *us;		/* User screen */
 EXTERN WIN *st;		/* Status Line */
 
-EXTERN int real_uid;	/* Real uid */
-EXTERN int real_gid;	/* Real gid */
-EXTERN int eff_uid;	/* Effective uid */
-EXTERN int eff_gid;	/* Effective gid */
 EXTERN short terminal;	/* terminal type */
 EXTERN time_t online;	/* Time online in minutes */
 EXTERN long linespd;	/* Line speed */
@@ -102,7 +100,6 @@ EXTERN char scr_passwd[33]; /* Password to use with script */
 EXTERN char termtype[32];  /* Value of getenv("TERM"); */
 EXTERN char *dial_tty;     /* tty to use. */
 
-EXTERN int portuid, portgid; /* Uid and Gid of /dev/ttyS... */
 EXTERN char *dial_name;	    /* System we're conneced to */
 EXTERN char *dial_number;   /* Number we've dialed. */
 EXTERN char *dial_user;     /* Our username there */
@@ -170,6 +167,7 @@ void vdodflsave(void);	/* fmg - need it */
 int  domacsave(void);	/* fmg - need it */
 int  loadconv(char *);	/* jl */
 int  saveconv(char *);	/* jl */
+int  speed_valid(unsigned int);
 
 /* Prototypes from file: common.c */
 char *pfix_home( char *s);
@@ -205,13 +203,12 @@ char *get_port(char *);
 int help(void);
 
 /* Prototypes from file: ipc.c */
-int check_io(int fd1, int fd2, int tmout, char *buf, int *buflen);
+int check_io(int fd1, int fd2, int tmout, char *buf, int buf_size, int *bytes_red);
 int keyboard(int cmd, int arg);
 
 /* Prototypes from file: keyserv.c */
 void handler(int dummy);
 void sendstr(char *s);
-int main(int argc, char **argv);
 
 /* Prototypes from file: main.c */
 extern time_t old_online;
@@ -241,19 +238,16 @@ int  find_next(WIN *w, WIN *w_hist, int hit_line, wchar_t *look,
 const wchar_t *upcase(wchar_t *dest, wchar_t *src);
 wchar_t *StrStr(wchar_t *str1, wchar_t *str2, int case_matters);
 
+void do_iconv(char **inbuf, size_t *inbytesleft,
+              char **outbuf, size_t *outbytesleft);
+int  using_iconv(void);
+
 /* Prototypes from file: rwconf.c */
 int writepars(FILE *fp, int all);
 int writemacs(FILE *fp);
 int readpars(FILE *fp, enum config_type conftype);
 int readmacs(FILE *fp, int init); /* fmg */
 
-#if 0 /* Only if you don't have it already */
-char *strtok(char *s, char *delim);
-#endif
-#ifdef _SVR2
-int dup2(int from, int to);
-#endif
-
 /* Prototypes from file: sysdep1.c */
 void m_sethwf(int fd, int on);
 void m_dtrtoggle(int fd, int sec);
@@ -275,17 +269,6 @@ int  m_wait(int *st);
 void getrowcols(int *rows, int *cols);
 int  setcbreak(int mode);
 void enab_sig(int onoff, int intrchar);
-void drop_all_privs( void);
-void drop_privs( void);
-void set_privs( void);
-FILE *sfopen(const char *file, const char *mode);
-
-#if 0 /* Only if you don't have it already */
-char *strtok(char *s, char *delim);
-#endif
-#ifdef _SVR2
-int dup2(int from, int to);
-#endif
 
 /* Prototypes from file: updown.c */
 void updown(int what, int nr );
@@ -310,7 +293,8 @@ extern int io_pending, pendingkeys;
 void domacros(void);
 
 
-
+void lockfile_create(void);
+void lockfile_remove(void);
 
 
 
