@@ -38,9 +38,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: rwconf.c,v 1.12 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -86,8 +83,8 @@ struct pars mpars[] = {
   { "/usr/bin/rz -vv -b -E",	0,   "pprog4" },
   { "/usr/bin/rb -vv",		0,   "pprog5" },
   { "/usr/bin/rx -vv",		0,   "pprog6" },
-  { "/usr/bin/kermit -i -l %l -s", 0, "pprog7" },
-  { "/usr/bin/kermit -i -l %l -r", 0, "pprog8" },
+  { "/usr/bin/kermit -i -l %l -b %b -s", 0, "pprog7" },
+  { "/usr/bin/kermit -i -l %l -b %b -r", 0, "pprog8" },
 #else
   /* Most sites have this in /usr/local, except Linux. */
   { "/usr/local/bin/sz -vv",	0,   "pprog1" },
