@@ -32,9 +32,6 @@
 
 #include <stdarg.h>
 
-#include "rcsid.h"
-RCSID(PKG_VER "$Id: script.c,v 1.14 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -202,11 +199,9 @@ char *getword(char **s)
         case 'b':
           buf[idx++] = '\b';
           break;
-#ifndef _HPUX_SOURCE
         case 'a':
           buf[idx++] = '\a';
           break;
-#endif
         case 'f':
           buf[idx++] = '\f';
           break;
