@@ -22,13 +22,13 @@
  * acme 25.02.98 i18n
  * js&jl 04.98	the better filename selection window
  */
+
+#include <poll.h>
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: updown.c,v 1.9 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -36,6 +36,7 @@ RCSID("$Id: updown.c,v 1.9 2007-10-10 20:18:20 al-guest Exp $")
 /*#define LOG_XFER	  debugging option to log all output of rz/sz
  */
 static int udpid;
+static int script_running;
 
 /*
  * Change to a directory.
@@ -51,17 +52,13 @@ static int mcd(char *dir)
     if (*dir == 0)
       return 0;
     init = 1;
-#if defined (_COH3) || defined(NeXT)
-    getwd(odir);
-#else
     getcwd(odir, 255);
-#endif
   }
   if (*dir == 0) {
     chdir(odir);
     return 0;
   }
-  
+
   if (*dir != '/') {
     snprintf(buf, sizeof(buf), "%s/%s", homedir, dir);
     dir = buf;
@@ -85,6 +82,7 @@ static void udcatch(int dummy)
   signal(SIGINT, udcatch);
   if (udpid)
     kill((pid_t)udpid, SIGKILL);
+  script_running = 0;
 }
 
 /*
@@ -274,12 +272,12 @@ void updown(int what, int nr)
         if (pipefd[1] != 2)
           close(pipefd[1]);
       }
+
+      lockfile_remove();
+
       for (n = 1; n < _NSIG; n++)
         signal(n, SIG_DFL);
 
-      set_privs();
-      setgid((gid_t)real_gid);
-      setuid((uid_t)real_uid);
       fastexec(translate(cmdline));
       exit(1);
     default: /* Parent */
@@ -344,6 +342,8 @@ void updown(int what, int nr)
   if (win == (WIN *)0)
     mc_wreturn();
 
+  lockfile_create();
+
   /* MARK updated 02/17/94 - Flush modem port before displaying READY msg */
   /* because a BBS often displays menu text right after a download, and we */
   /* don't want the modem buffer to be lost while waiting for key to be hit */
@@ -375,6 +375,32 @@ void updown(int what, int nr)
     mc_wclose(win, 1);
 }
 
+void lockfile_remove(void)
+{
+  if (lockfile[0])
+    unlink(lockfile);
+}
+
+void lockfile_create(void)
+{
+  int fd, n;
+  char buf[81];
+
+  if (!lockfile[0])
+    return;
+
+  n = umask(022);
+  /* Create lockfile compatible with UUCP-1.2 */
+  if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
+    werror(_("Cannot create lockfile!"));
+  } else {
+    snprintf(buf, sizeof(buf),  "%05d minicom %.20s\n", (int)getpid(), username);
+    write(fd, buf, strlen(buf));
+    close(fd);
+  }
+  umask(n);
+}
+
 /*
  * Run kermit. Used to do this in the main window, but newer
  * versions of kermit are too intelligent and just want a tty
@@ -382,10 +408,13 @@ void updown(int what, int nr)
  */
 void kermit(void)
 {
-  int status;
-  int pid, n;
-  char buf[81];
-  int fd;
+  int status, pid, n;
+  char *kermit_path = translate(P_KERMIT);
+
+  if (!kermit_path || !*kermit_path) {
+    werror("No kermit path defined!");
+    return;
+  }
 
   /* Clear screen, set keyboard modes etc. */
   mc_wleave();
@@ -396,17 +425,15 @@ void kermit(void)
       werror(_("Out of memory: could not fork()"));
       return;
     case 0: /* Child */
+      close(portfd);
+
       /* Remove lockfile */
-      set_privs();
-      if (lockfile[0])
-        unlink(lockfile);
-      setgid((gid_t)real_gid);
-      setuid((uid_t)real_uid);
+      lockfile_remove();
 
       for (n = 0; n < _NSIG; n++)
         signal(n, SIG_DFL);
 
-      fastexec(translate(P_KERMIT));
+      fastexec(kermit_path);
       exit(1);
     default: /* Parent */
       break;
@@ -418,21 +445,8 @@ void kermit(void)
   mc_wreturn();
 
   /* Re-create lockfile */
-  if (lockfile[0]) {
-    set_privs();
-    n = umask(022);
-    /* Create lockfile compatible with UUCP-1.2 */
-    if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
-      werror(_("Cannot re-create lockfile!"));
-    } else {
-      chown(lockfile, (uid_t)real_uid, (gid_t)real_gid);
-      snprintf(buf, sizeof(buf),  "%05d minicom %.20s\n", (int)getpid(), username);
-      write(fd, buf, strlen(buf));
-      close(fd);
-    }
-    umask(n);
-    drop_privs();
-  }
+  lockfile_create();
+
   m_flush(portfd);
   port_init();
 }
@@ -511,11 +525,12 @@ int mc_setenv(const char *name, const char *value)
 void runscript(int ask, const char *s, const char *l, const char *p)
 {
   int status;
-  int n;
+  int n, i;
   int pipefd[2];
   char buf[81];
   char scr_lines[5];
   char cmdline[128];
+  struct pollfd fds[2];
   char *ptr;
   WIN *w;
   int done = 0;
@@ -589,7 +604,7 @@ void runscript(int ask, const char *s, const char *l, const char *p)
     st = NULL;
   }
   scriptname(scr_name);
-  
+
   pipe(pipefd);
 
   if (mcd(P_SCRIPTDIR) < 0)
@@ -615,9 +630,6 @@ void runscript(int ask, const char *s, const char *l, const char *p)
       for (n = 1; n < _NSIG; n++)
 	signal(n, SIG_DFL);
 
-      set_privs();
-      setgid((gid_t)real_gid);
-      setuid((uid_t)real_uid);
       mc_setenv("LOGIN", scr_user);
       mc_setenv("PASS", scr_passwd);
       mc_setenv("TERMLIN", scr_lines);	/* jl 13.09.97 */
@@ -630,16 +642,27 @@ void runscript(int ask, const char *s, const char *l, const char *p)
   enab_sig(1, 0);	       /* But enable SIGINT */
   signal(SIGINT, udcatch);
   close(pipefd[1]);
-  
+
   /* pipe output from "runscript" program to terminal emulator */
-  while ((n = read(pipefd[0], buf, 80)) > 0) {
-    ptr = buf;
-    while (n--)
-      vt_out(*ptr++);
-    timer_update();
-    mc_wflush();
-  }
-  
+  fds[0].fd     = pipefd[0]; /* runscript */
+  fds[0].events = POLLIN;
+  fds[1].fd     = STDIN_FILENO; /* stdin */
+  fds[1].events = POLLIN;
+  script_running = 1;
+  while (script_running && poll(fds, 2, -1) > 0)
+    for (i = 0; i < 2; i++)
+      if ((fds[i].revents & POLLIN)
+          && (n = read(fds[i].fd, buf, sizeof(buf)-1)) > 0) {
+        ptr = buf;
+        while (n--)
+          if (i)
+            vt_send(*ptr++);
+          else
+            vt_out(*ptr++);
+        timer_update();
+        mc_wflush();
+      }
+
   /* Collect status, and clean up. */
   m_wait(&status);
   enab_sig(0, 0);
@@ -668,7 +691,7 @@ int paste_file(void)
   const int ldelay = 1;      /* hardcoded 1 ms */
   char buf[128] = "";
   char *ptr;
-  int blen;
+  int bytes_read;
   unsigned long bdone = 0;
   int x;
 
@@ -681,12 +704,12 @@ int paste_file(void)
 
   while (fgets(line, sizeof(line), fp)) {
     /* Check for I/O or timer. */
-    x = check_io(portfd_connected, 0, 1000, buf, &blen);
+    x = check_io(portfd_connected, 0, 1000, buf, sizeof(buf), &bytes_read);
 
     /*  Send data from the modem to the screen. */
     if ((x & 1)) {
       ptr = buf;
-      while (blen-- > 0) {
+      while (bytes_read-- > 0) {
 	if (P_PARITY[0] == 'M' || P_PARITY[0] == 'S')
 	  *ptr &= 0x7f;
 	vt_out(*ptr++);
