@@ -20,15 +20,10 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: util.c,v 1.8 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
 
-static int not_suid = -1;
-
 /*
  * A modified version of the getargs routine.
  */
@@ -179,115 +174,6 @@ int fastsystem(char *cmd, char *in, char *out, char *err)
   return -1;
 }
 
-/* Drop all priviliges (irreversable). */
-void drop_all_privs(void)
-{
-#ifdef HAS_REUID
-  /* Regain privs needed to drop privs :) */
-  setregid(real_gid, eff_gid);
-  setreuid(real_uid, eff_uid);
-#endif
-
-  /* Drop it. */
-  setgid(real_gid);
-  setuid(real_uid);
-  eff_uid = real_uid;
-  eff_gid = real_gid;
-  not_suid = 1;
-}
-
-/* Drop priviliges (swap uid's) */
-void drop_privs(void)
-{
-#ifdef HAS_REUID
-  setregid(eff_gid, real_gid);
-  if (setreuid(eff_uid, real_uid) < 0)
-    fprintf(stderr, _("minicom: cannot setreuid(%d, %d)\n"), eff_uid, real_uid);
-  not_suid = 1;
-#endif
-}
-
-/* Set priviliges (swap uid's) */
-void set_privs(void)
-{
-#ifdef HAS_REUID
-  setregid(real_gid, eff_gid);
-  if (setreuid(real_uid, eff_uid) < 0)
-    fprintf(stderr, _("minicom: cannot setreuid(%d, %d)\n"), real_uid, eff_uid);
-  not_suid = 0;
-#endif
-}
-
-/* Safe fopen for suid programs. */
-FILE *sfopen(const char *file, const char *mode)
-{
-#ifdef HAS_REUID
-  int saved_errno;
-#else
-  char *p, buf[128];
-  struct stat stt;
-  int do_chown = 0;
-#endif
-  FILE *fp;
-
-  /* Initialize. */
-  if (not_suid < 0)
-    not_suid = (real_uid == eff_uid);
-
-  /* If we're not running set-uid at the moment just open the file. */
-  if (not_suid)
-    return fopen(file, mode);
-
-#ifdef HAS_REUID
-  /* Just drop priviliges before opening the file. */
-  drop_privs();
-  fp = fopen(file, mode);
-  saved_errno = errno;
-  set_privs();
-  errno = saved_errno;
-  return fp;
-#else
-
-  /* Read access? */
-  if (strcmp(mode, "r") == 0) {
-    if (access(file, R_OK) < 0)
-      return NULL;
-    return fopen(file, mode);
-  }
-
-  /* Write access test. */
-  if (stat(file, &stt) == 0) {
-    if (access(file, W_OK) < 0)
-      return NULL;
-  } else {
-    /* Test if directory is writable. */
-    strncpy(buf, file, sizeof(buf));
-    if ((p = strrchr(buf, '/')) == NULL)
-      strcpy(buf, ".");
-    else
-      *p = '\0';
-    if (access(buf, W_OK) < 0)
-      return NULL;
-    do_chown = 1;
-  }
-
-  /* Now open/create the file. */
-  if ((fp = fopen(file, mode)) == NULL)
-    return fp;
-  if (!do_chown)
-    return fp;
-
-#ifndef HAS_FCHOWN
-  /* There is a security hole / race condition here. */
-  chown(file, (uid_t)real_uid, (gid_t)real_gid);
-#else
-  /* But this is safe. */
-  fchown(fileno(fp), (uid_t)real_uid, (gid_t)real_gid);
-#endif /* HAS_FCHOWN */
-  return fp;
-#endif /* HAS_REUID */
-}
-
 /*
  * Get next port from a space-, comma-, colon-, or semi-colon-separated
  * list (we're easy :-)) in a PARS_VAL_LEN length string.
