@@ -26,9 +26,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: vt100.c,v 1.14 2007-10-10 20:18:20 al-guest Exp $")
-
 #include "port.h"
 #include "minicom.h"
 #include "vt100.h"
@@ -61,7 +58,6 @@ struct escseq {
 
 /* Escape sequences for different terminal types. */
 static struct escseq vt_keys[] = {
-#ifndef _DGUX_SOURCE
   { K_F1,	"OP",	"OP",	"OP" },
   { K_F2,	"OQ",	"OQ",	"OQ" },
   { K_F3,	"OR",	"OR",	"OR" },
@@ -85,31 +81,6 @@ static struct escseq vt_keys[] = {
   { K_INS,	"[2~",	"[2~",	"[@" },
   { K_DEL,	"[3~",	"[3~",	"\177" },
   { 0,		NULL,	NULL,	NULL }
-#else
-  { K_F1,      "[17~", "[17~", "OP" },
-  { K_F2,      "[18~", "[18~", "OQ" },
-  { K_F3,      "[19~", "[19~", "OR" },
-  { K_F4,      "[20~", "[20~", "OS" },
-  { K_F5,      "[21~", "[21~", "OT" },
-  { K_F6,      "[23~", "[23~", "OU" },
-  { K_F7,      "[24~", "[24~", "OV" },
-  { K_F8,      "[25~", "[25~", "OW" },
-  { K_F9,      "[26~", "[26~", "OX" },
-  { K_F10,     "[28~", "[28~", "OY" },
-  { K_F11,     "[29~", "[29~", "OZ" },
-  { K_F12,     "[31~", "[31~", "OA" },
-  { K_HOME,    "OP",   "OP",   "[H" },
-  { K_PGUP,    "OQ",   "OQ",   "[V" },
-  { K_UP,      "[A",   "OA",   "[A" },
-  { K_LT,      "[D",   "OD",   "[D" },
-  { K_RT,      "[C",   "OC",   "[C" },
-  { K_DN,      "[B",   "OB",   "[B" },
-  { K_END,     "OR",   "OR",   "[Y" },
-  { K_PGDN,    "OS",   "OS",   "[U" },
-  { K_INS,     "[1~",  "[1~",  "[@"  },
-  { K_DEL,     "[3~",  "[3~",  "\177" },
-  { 0,         NULL,   NULL,   NULL }
-#endif
 };
 
 /* Two tables for user-defined character map conversions.
@@ -1038,21 +1009,27 @@ void vt_out(int ch)
     case 0: /* Normal character */
       if (vt_docap == 1)
         fputc(P_CONVCAP[0] == 'Y' ? vt_inmap[c] : c, capfp);
+      if (!using_iconv()) {
       c = vt_inmap[c];    /* conversion 04.09.97 / jl */
 #if TRANSLATE
       if (vt_type == VT100 && vt_asis == 0)
-	c = vt_trans[vt_charset][c];
+        c = vt_trans[vt_charset][c];
 #endif
+      }
       /* FIXME: This is wrong, but making it right would require
        * removing all the 8-bit mapping features. Assuming the locale
        * is 8-bit, the character should not be changed by mapping to
        * wchar and back; if the locale is multibyte, there is no hope
        * of getting it right anyway. */
-      one_mbtowc (&wc, (char *)&c, 1); /* returns 1 */
-      if (vt_insert)
-        mc_winschar2(vt_win, wc, 1);
-      else
-        mc_wputc(vt_win, wc);
+      if (!using_iconv()) {
+        one_mbtowc (&wc, (char *)&c, 1); /* returns 1 */
+        if (vt_insert)
+          mc_winschar2(vt_win, wc, 1);
+        else
+          mc_wputc(vt_win, wc);
+      } else {
+        mc_wputc(vt_win, c);
+      }
       break;
     case 1: /* ESC seen */
       state1(c);
