@@ -22,9 +22,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: window.c,v 1.21 2008-02-06 21:49:55 al-guest Exp $")
-
 #include <limits.h>
 #include <stdarg.h>
 #include <wchar.h>
@@ -387,11 +384,11 @@ static void _write(wchar_t c, int doit, int x, int y, char attr, char color)
         _setattr(attr, color);
       }
       x0 = x; y0 = y; attr0 = attr; color0 = color; c0 = c;
-      if ((attr & XA_ALTCHARSET) != 0)
-	outchar((char)c);
+      if (using_iconv() || (attr & XA_ALTCHARSET) != 0)
+        outchar((char)c);
       else {
-	char buf[MB_LEN_MAX];
-	size_t i, len;
+        char buf[MB_LEN_MAX];
+        size_t i, len;
 
         len = one_wctomb(buf, c);
         for (i = 0; i < (size_t)len; i++)
