@@ -20,9 +20,6 @@
 #include <config.h>
 #endif
 
-#include "rcsid.h"
-RCSID("$Id: wkeys.c,v 1.10 2007-12-31 12:38:24 al-guest Exp $")
-
 #include <strings.h>
 
 #include "port.h"
@@ -50,11 +47,7 @@ int io_pending = 0;
 static const char *func_key[] = {
   "", "k1", "k2", "k3", "k4", "k5", "k6", "k7", "k8", "k9", "k0",
   "kh", "kP", "ku", "kl", "kr", "kd", "kH", "kN", "kI", "kD",
-#ifdef _DGUX_SOURCE
-  "kA", "kB", NULL };
-#else
   "F1", "F2", NULL };
-#endif
 #if KEY_KLUDGE
 /*
  * A VERY DIRTY HACK FOLLOWS:
